/******************************************************************************
 *
 * Sugar for Java 1.2.0
 * Copyright (C) 2001-2004  Dipl.-Inf. (FH) Johann Nepomuk Loefflmann,
 * All Rights Reserved, http://www.jonelo.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author jonelo@jonelo.de
 *
 *****************************************************************************/

package jonelo.sugar.util;
import java.util.*;

public class ExtendedGregorianCalendar extends java.util.GregorianCalendar {

  public static final int
  LAST                            = -1,
  FIRST                           = 1,
  SECOND                          = 2,
  THIRD                           = 3,
  FOURTH                          = 4;

   public static String
     ERROR_1583               = "Year must be 1583 or later";

    /** Creates a new instance of ExtendedGregorianCalendar */
    public ExtendedGregorianCalendar() {
    }

  /*
   * From chapter 4 of "Astronomical Formulae for Calculators" 2nd
   * edition; by Jean Meeus; publisher: Willmann-Bell Inc.,
   * ISBN 0-943396-01-8 ...
   *
   *                          Date of Easter.
   *
   * The method used below has been given by Spencer Jones in his
   * book "General Astronomy" (pages 73-74 of the edition of 1922).
   * It has been published again in the "Journal of the British
   * Astronomical Association", Vol.88, page 91 (December 1977)
   * where it is said that it was devised in 1876 and appeared in
   * the Butcher's "Ecclesiastical Calendar."
   *
   * Unlike the formula given by Guass, this method has no exception
   * and is valid for all years in the Gregorian calendar, that is
   * from the year 1583 on.
   *
   * [...text omitted...]
   *
   * The extreme dates of Easter are March 22 (as in 1818 and 2285)
   * and April 25 (as in 1886, 1943, 2038).
   * ===============================================================
   *
   * The following Modula-2 code by Greg Vigneault, April 1993.
   *
   * Converted To Pascal by Kerry Sokalsky
   *
   * Converted To Java by Johann N. Loefflmann
   */
  /**
   * setzt den Ostertag des bergebenen Jahres
   * @param year das Jahr in dessen der Ostertag zu bestimmen ist.
   */
  public void setEaster(int year) throws Exception
  {
    int
    a, b, c,
    d, e, f,
    g, h, i,
    k, l, m,
    n, p;

    if ((year < 1583) || (year < 0))
    throw new Exception(ERROR_1583);

    a = year % 19;
    b = year / 100;
    c = year % 100;
    d = b / 4;
    e = b % 4;
    f = (b + 8) / 25;
    g = (b - f + 1) / 3;
    h = (19 * a + b - d - g + 15) % 30;
    i = c / 4;
    k = c % 4;
    l = (32 + 2 * e + 2 * i - h - k) % 7;
    m = (a + 11 * h + 22 * l) / 451;
    p = (h + l - 7 * m + 114);
    n = p / 31;                  // n = month number 3 or 4
    p = (p % 31) + 1;            // p = day in month

    set(Calendar.DATE,  p);
    set(Calendar.MONTH, n-1);
    set(Calendar.YEAR,  year);
    setTime(getTime());
  }


  // LAST, FIRST, SECOND...
  public void setNthDow(int n, int dow)
  {
    set(DAY_OF_WEEK, dow);
    set(DAY_OF_WEEK_IN_MONTH, n);
   // setTime(getTime());
  }

  public void setNthDow(int n, int dow, int month)
  {
    set(Calendar.MONTH, month);
    setNthDow(n, dow);
  }

  public void setNthDow(int n, int dow, int month, int year)
  {
    set(Calendar.YEAR, year);
    setNthDow(n, dow, month);
  }

  /**
   * setzt den vorhergehenden Wochentag dow
   * (wenn Tag im Objekt bereits dow ist, wird nichts gemacht)
   */
  public void setPrevDow(int dow)
  {
    while (get(Calendar.DAY_OF_WEEK) != dow)
    {
      add(Calendar.DATE, -1);
//      setTime(getTime());
    }
  }

  /**
   * setzt den nachfolgenden Wochentag dow
   * (wenn Tag im Objekt bereits dow ist, wird nichts gemacht)
   */
  public void setNextDow(int dow)
  {
    while (get(Calendar.DAY_OF_WEEK) != dow)
    {
      add(Calendar.DATE, +1);
//      setTime(getTime());
    }
  }

  // setzt den 1. Adventssonntag
  public void setAdvent(int year)
  {
    // Referenzdatum setzen
    set(year,Calendar.DECEMBER,24);

    // 1. Sonntag vor dem Referenzdatum suchen
    setPrevDow(SUNDAY);

    // 3 weitere Wochen (21 Tage) zurckgehen
    // (insgesamt ist es dann die 4. Woche vor Weihnachten)
    add(Calendar.DATE , -21);
    // setTime(getTime());
  }


  /**
   * http://www.jguru.com/forums/view.jsp?EID=489372
   * Calculates the number of days between two calendar days in a manner
   * which is independent of the Calendar type used.
   *
   * @param d1    The first date.
   * @param d2    The second date.
   *
   * @return      The number of days between the two dates.  Zero is
   *              returned if the dates are the same, one if the dates are
   *              adjacent, etc.  The order of the dates
   *              does not matter.
   * the value returned is always >= 0.
   *              If Calendar types of d1 and d2
   *              are different, the result may not be accurate.
   */
  public static int getDaysBetween (java.util.Calendar d1, java.util.Calendar d2) {
    int factor=1;
    if (d1.after(d2)) {  // swap dates so that d1 is start and d2 is end
        java.util.Calendar swap = d1;
        d1 = d2;
        d2 = swap;
        factor=-1;
    }
    int days = d2.get(java.util.Calendar.DAY_OF_YEAR) -
               d1.get(java.util.Calendar.DAY_OF_YEAR);
    int y2 = d2.get(java.util.Calendar.YEAR);
    if (d1.get(java.util.Calendar.YEAR) != y2) {
        d1 = (java.util.Calendar) d1.clone();
        do {
            days += d1.getActualMaximum(java.util.Calendar.DAY_OF_YEAR);
            d1.add(java.util.Calendar.YEAR, 1);
        } while (d1.get(java.util.Calendar.YEAR) != y2);
    }
    return factor*days;
  } // getDaysBetween()

}
